/*
pbcZ.c
*/

#include "oth.h"
#include "complex.h"
#include "oth_prototype.h"

void copyZpbc(d_complex_t *v)
{
	assert(Pbc.zpbc);

	// Ez
	for (int i = i0Ez - 1; i <= i1Ez + 1; i++) {
	for (int j = j0Ez - 1; j <= j1Ez + 1; j++) {
		//assert(NEZ(i, j,    - 1) >= 0 && NEZ(i, j,    - 1) < NE);
		//assert(NEZ(i, j, Nz    ) >= 0 && NEZ(i, j, Nz    ) < NE);
		v[NEZ(i, j,    - 1)] = v[NEZ(i, j, Nz - 1)];
		v[NEZ(i, j, Nz + 0)] = v[NEZ(i, j,      0)];
	}
	}

	// Ex
	for (int i = i0Ex - 1; i <= i1Ex + 1; i++) {
	for (int j = j0Ex - 1; j <= j1Ex + 1; j++) {
		//assert(NEX(i, j,    - 1) >= 0 && NEX(i, j,    - 1) < NE);
		//assert(NEX(i, j, Nz + 1) >= 0 && NEX(i, j, Nz + 1) < NE);
		v[NEX(i, j,    - 1)] = v[NEX(i, j, Nz - 1)];
		v[NEX(i, j, Nz + 1)] = v[NEX(i, j,      1)];
	}
	}

	// Ey
	for (int i = i0Ey - 1; i <= i1Ey + 1; i++) {
	for (int j = j0Ey - 1; j <= j1Ey + 1; j++) {
		//assert(NEY(i, j,    - 1) >= 0 && NEY(i, j,    - 1) < NE);
		//assert(NEY(i, j, Nz + 1) >= 0 && NEY(i, j, Nz + 1) < NE);
		v[NEY(i, j,    - 1)] = v[NEY(i, j, Nz - 1)];
		v[NEY(i, j, Nz + 1)] = v[NEY(i, j,      1)];
	}
	}
}

void factorZpbc(d_complex_t *v)
{
	assert(Pbc.zpbc);
	int64_t n;
	int k;
	const d_complex_t fp = Pbc.fz;
	const d_complex_t fm = d_inv(fp);

	// Ez
	for (int i = i0Ez - 1; i <= i1Ez + 1; i++) {
	for (int j = j0Ez - 1; j <= j1Ez + 1; j++) {
		k =    - 1;
		if ((k >= k0Ez - 1) && (k <= k1Ez + 1)) {  // MPI
			n = NEZ(i, j, k);
			v[n] = d_mul(v[n], fm);
		}

		k = Nz + 0;
		if ((k >= k0Ez - 1) && (k <= k1Ez + 1)) {  // MPI
			n = NEZ(i, j, k);
			v[n] = d_mul(v[n], fp);
		}
	}
	}

	// Ex
	for (int i = i0Ex - 1; i <= i1Ex + 1; i++) {
	for (int j = j0Ex - 1; j <= j1Ex + 1; j++) {
		k =    - 1;
		if ((k >= k0Ex - 1) && (k <= k1Ex + 1)) {  // MPI
			n = NEX(i, j, k);
			v[n] = d_mul(v[n], fm);
		}

		k = Nz + 1;
		if ((k >= k0Ex - 1) && (k <= k1Ex + 1)) {  // MPI
			n = NEX(i, j, k);
			v[n] = d_mul(v[n], fp);
		}
	}
	}

	// Ey
	for (int i = i0Ey - 1; i <= i1Ey + 1; i++) {
	for (int j = j0Ey - 1; j <= j1Ey + 1; j++) {
		k =    - 1;
		if ((k >= k0Ey - 1) && (k <= k1Ey + 1)) {  // MPI
			n = NEY(i, j, k);
			v[n] = d_mul(v[n], fm);
		}

		k = Nz + 1;
		if ((k >= k0Ey - 1) && (k <= k1Ey + 1)) {  // MPI
			n = NEY(i, j, k);
			v[n] = d_mul(v[n], fp);
		}
	}
	}
}
